# Stochastic Automatic Differentiation

This toolbox contains a C++ implementation for automatic differentiation of stochastic functions. The functions are demonstrated using several problems. Visualization is done in Python. The default parameters are given to reproduce the figures in the paper "On automatic differentiation of stochastic programs" by Montefusco, Niemann and Weiser.

## Birth-Death Process
Implementaions of a chemical reaction network modeling a birth-death process. If not stated differently, all implementations use Gillespie's stochastic simulation algorithm. This is a linear chemical reaction network.

### `BirthDeath.cpp`
Stochastic automatic differentiation with respect to reaction rate constants $\gamma_1$ and $\gamma_2$ of the birth-death process.

The code offers an adaptive jump probability that scales with the expected remaining number of steps of the Gillespie algorithm which can be activated with the flag `--adaptive`; by default off. Note: only tested with `T_max = 100`.

The following parameters can be adjusted in the code:

The following parameters can be adjusted in the code:

* `const int init_state` Initial state of the chemical reaction network. Default is `0`.
* `const double volume` Scaling factor. Default is `10`.
* `const double T_max` Time to simulate. Default is `100`.
* `const int num_steps = 500` Number of time steps for discretization. Default is `500`.
* `const int MC` Number of Monte Carlo samples. Default is `1e5`.
* `const double w` Jump probability. Default is `0.01`.
* `const std::array<double, 2> gamma` Birith and death rates. Default is `{0.2, 0.1}`.

### `BirthDeath_FD.cpp`
Finite differences differentiation with respect to reaction rate constants $\gamma_1$ and $\gamma_2$ of the birth-death process. The code offers the flag `--correlated-sampling` to activate correlated sampling; by default off.

The following parameters can be adjusted in the code:

* `const int init_state` Initial state of the chemical reaction network. Default is `0`.
* `const double volume` Scaling factor. Default is `10`.
* `const double T_max` Time to simulate. Default is `100`.
* `const int num_steps = 500` Number of time steps for discretization. Default is `500`.
* `const int MC` Number of Monte Carlo samples. Default is `1e5`.
* `const double h` Finite differences step size. Default is `0.01`.
* `const std::array<double, 2> gamma` Birith and death rates. Default is `{0.2, 0.1}`.

### `BirthDeath_ODE.cpp`
Euler implementation of the chemical reaction network for $N\to\infty$ molecules. The code uses the stochastic automatic differentiation toolbox to compute the parametric derivatives.

The following parameters can be adjusted in the code:

* `const int init_state` Initial state of the chemical reaction network. Default is `0`.
* `const double volume` Scaling factor. Default is `10`.
* `const double T_max` Time to simulate. Default is `100`.
* `const int num_steps = 500` Number of time steps for discretization. Default is `500`.

## Binding-Unbinding Process
Implementaions of a chemical reaction network modeling binding and unbinding of two molecules. If not stated differently, all implementations use Gillespie's stochastic simulation algorithm. This is a nonlinear chemical reaction network.

### `BindUnbind.cpp`
Stochastic automatic differentiation with respect to reaction rate constants $\gamma_1$ and $\gamma_2$ of the binding-unbinding process.

The code offers an adaptive jump probability that scales with the expected remaining number of steps of the Gillespie algorithm which can be activated with the flag `--adaptive`; by default off. Note: only tested with `T_max = 25`.

The following parameters can be adjusted in the code:

* `const std::array<int, 3> init_state` Initial state of the chemical reaction network. Default is `{100, 100, 0}`.
* `const double volume` Scaling factor. Default is `100`.
* `const double T_max` Time to simulate. Default is `25`.
* `const int num_steps = 500` Number of time steps for discretization. Default is `500`.
* `const int MC` Number of Monte Carlo samples. Default is `1e5`.
* `const double w` Jump probability. Default is `0.01`.
* `const std::array<double, 2> gamma` Binding and unbinding rates. Default is `{0.1, 0.2}`.

### `BindUnbind_FD.cpp`
Finite differences differentiation with respect to reaction rate constants $\gamma_1$ and $\gamma_2$ of the binding-unbinding process. The code offers the flag `--correlated-sampling` to activate correlated sampling; by default off.

The following parameters can be adjusted in the code:

* `const std::array<int, 3> init_state` Initial state of the chemical reaction network. Default is `{100, 100, 0}`.
* `const double volume` Scaling factor. Default is `100`.
* `const double T_max` Time to simulate. Default is `25`.
* `const int num_steps = 500` Number of time steps for discretization. Default is `500`.
* `const int MC` Number of Monte Carlo samples. Default is `1e5`.
* `const double h` Finite differences step size. Default is `0.01`.
* `const std::array<double, 2> gamma` Binding and unbinding rates. Default is `{0.1, 0.2}`.

### `BindUnbind_ODE.cpp`
Euler implementation of the chemical reaction network for $N\to\infty$ molecules. The code uses the stochastic automatic differentiation toolbox to compute the parametric derivatives.

The following parameters can be adjusted in the code:

* `const std::array<int, 3> init_state` Initial state of the chemical reaction network. Default is `{100, 100, 0}`.
* `const double volume` Scaling factor. Default is `100`.
* `const double T_max` Time to simulate. Default is `25`.
* `const int num_steps = 500` Number of time steps for discretization. Default is `500`.