#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Aug  3 10:36:24 2023

@author: Jan-Hendrik Niemann
"""


import numpy as np
import matplotlib.pyplot as plt


# Figure export
export_image = False

# %% Load data and used simulation settings

data_mean = np.genfromtxt('out_bindUnbind_mean.csv', delimiter=" ", dtype=float)
data_mean_FD = np.genfromtxt('out_bindUnbind_mean_FD.csv', delimiter=" ", dtype=float)
data_var = np.genfromtxt('out_bindUnbind_var.csv', delimiter=" ", dtype=float)
data_std = np.sqrt(data_var)
data_var_FD = np.genfromtxt('out_bindUnbind_var_FD.csv', delimiter=" ", dtype=float)
data_std_FD = np.sqrt(data_var_FD)
data_ODE = np.genfromtxt('out_bindUnbind_ODE.csv', delimiter=" ", dtype=float)

T_max = 25
time = np.linspace(0, T_max, data_mean.shape[0])

# %% Compute variances and standard deviations

var_1 = np.zeros((data_mean.shape[0], 3))  # Binding
var_2 = np.zeros((data_mean.shape[0], 3))  # Unbinding
for i in range(3):
    var_1[:, i] = data_var[:, 3 + i] + data_mean[:, 11 + i]**2 * data_var[:, 9] + data_mean[:, 9]**2 * data_var[:, 11 + i] + data_var[:, 9] * data_var[:, 11 + i] - 2 * (data_mean[:, 14 + i] - data_mean[:, 3 + i] * data_mean[:, 9] * data_mean[:, 11 + i])
    var_2[:, i] = data_var[:, 6 + i] + data_mean[:, 11 + i]**2 * data_var[:, 10] + data_mean[:, 10]**2 * data_var[:, 11 + i] + data_var[:, 10] * data_var[:, 11 + i] - 2 * (data_mean[:, 17 + i] - data_mean[:, 6 + i] * data_mean[:, 10] * data_mean[:, 11 + i])

# Compute standard deviation
std_1 = np.sqrt(var_1)  # Binding
std_2 = np.sqrt(var_2)  # Unbinding

# %% Plot

fig, axs = plt.subplots(3, sharex=True)
for i in range(3):
    axs[i].plot(time, data_ODE[:, i], color='k')
    axs[i].plot(time, data_mean[:, i], label=f'SAD $X_{i+1}(t)$', ls='--')
    axs[i].fill_between(time, data_mean[:, i] - data_std[:, i], data_mean[:, i] + data_std[:, i], alpha=0.25)
    axs[i].plot(time, data_mean_FD[:, i], label=f'FD $X_{i+1}(t)$', ls=':')
    axs[i].fill_between(time, data_mean_FD[:, i] - data_std_FD[:, i], data_mean_FD[:, i] + data_std_FD[:, i], alpha=0.25)
axs[0].legend()
axs[0].set_title('$E(p, t)$')
plt.xlabel('Time $t$')
fig.tight_layout()

fig, axs = plt.subplots(3, 2, sharex=True)
for i in range(3):

    # axs[i, 0].plot(time, data_ODE[:, i + 3], color='k')
    axs[i, 0].plot(time, data_mean[:, i + 3] - data_mean[:, 9] * data_mean[:, i + 11], label=f'SAD $X_{i+1}(t)$')
    axs[i, 0].fill_between(time, data_mean[:, i + 3] - data_mean[:, 9] * data_mean[:, i + 11] - std_1[:, i], data_mean[:, i + 3] - data_mean[:, 9] * data_mean[:, i + 11] + std_1[:, i], alpha=0.25, label='_nolegend_')
    axs[i, 0].plot(time, data_mean_FD[:, i + 3], label=f'FD $X_{i+1}(t)$')
    axs[i, 0].fill_between(time, data_mean_FD[:, i + 3] - data_std_FD[:, i + 3], data_mean_FD[:, i + 3] + data_std_FD[:, i + 3], alpha=0.25, label='_nolegend_')
    # axs[i, 1].plot(time, data_ODE[:, i + 6], color='k')
    axs[i, 1].plot(time, data_mean[:, i + 6] - data_mean[:, 10] * data_mean[:, i + 11], label=f'SAD $X_{i+1}(t)$')
    axs[i, 1].fill_between(time, data_mean[:, i + 6] - data_mean[:, 10] * data_mean[:, i + 11] - std_2[:, i], data_mean[:, i + 6] - data_mean[:, 9] * data_mean[:, i + 11] + std_2[:, i], alpha=0.25, label='_nolegend_')
    axs[i, 1].plot(time, data_mean_FD[:, i + 6], label=f'FD $X_{i+1}(t)$')
    axs[i, 1].fill_between(time, data_mean_FD[:, i + 6] - data_std_FD[:, i + 6], data_mean_FD[:, i + 6] + data_std_FD[:, i + 6], alpha=0.25, label='_nolegend_')

axs[0, 0].set_title(r'$\frac{\partial}{\partial\gamma_1}E(p, t)$', pad=15)
axs[2, 0].set_xlabel('Time $t$')
axs[0, 1].set_title(r'$\frac{\partial}{\partial\gamma_2}E(p, t)$', pad=15)
axs[2, 1].set_xlabel('Time $t$')
plt.figlegend(['SAD', 'FD'], ncols=2, loc=10, bbox_to_anchor=(0.535, 0.0))
fig.tight_layout()

# %% Figure export

fig, axs = plt.subplots(2, 2, sharex=True)
im_idx = -1
for i in range(3):

    if i == 1:
        continue
    else:
        im_idx += 1

    # axs[im_idx, 0].plot(time, data_ODE[:, i + 3], color='k')
    axs[im_idx, 0].plot(time, data_mean[:, i + 3] - data_mean[:, 9] * data_mean[:, i + 11], label=f'SAD $X_{i+1}(t)$')
    axs[im_idx, 0].fill_between(time, data_mean[:, i + 3] - data_mean[:, 9] * data_mean[:, i + 11] - std_1[:, i], data_mean[:, i + 3] - data_mean[:, 9] * data_mean[:, i + 11] + std_1[:, i], alpha=0.25, label='_nolegend_')
    axs[im_idx, 0].plot(time, data_mean_FD[:, i + 3], label=f'FD $X_{i+1}(t)$')
    axs[im_idx, 0].fill_between(time, data_mean_FD[:, i + 3] - data_std_FD[:, i + 3], data_mean_FD[:, i + 3] + data_std_FD[:, i + 3], alpha=0.25, label='_nolegend_')
    # axs[im_idx, 1].plot(time, data_ODE[:, i + 6], color='k')
    axs[im_idx, 1].plot(time, data_mean[:, i + 6] - data_mean[:, 10] * data_mean[:, i + 11], label=f'SAD $X_{i+1}(t)$')
    axs[im_idx, 1].fill_between(time, data_mean[:, i + 6] - data_mean[:, 10] * data_mean[:, i + 11] - std_2[:, i], data_mean[:, i + 6] - data_mean[:, 9] * data_mean[:, i + 11] + std_2[:, i], alpha=0.25, label='_nolegend_')
    axs[im_idx, 1].plot(time, data_mean_FD[:, i + 6], label=f'FD $X_{i+1}(t)$')
    axs[im_idx, 1].fill_between(time, data_mean_FD[:, i + 6] - data_std_FD[:, i + 6], data_mean_FD[:, i + 6] + data_std_FD[:, i + 6], alpha=0.25, label='_nolegend_')

axs[0, 0].set_title(r'$\frac{\partial}{\partial\gamma_1}E(p, t)$', pad=15)
axs[1, 0].set_xlabel('Time $t$')
axs[0, 1].set_title(r'$\frac{\partial}{\partial\gamma_2}E(p, t)$', pad=15)
axs[1, 1].set_xlabel('Time $t$')
plt.figlegend(['SAD', 'FD'], ncols=2, loc=10, bbox_to_anchor=(0.535, 0.0))
fig.tight_layout()

axs[0, 0].annotate('$\mathcal{S}_1$', xy=(0.05, 0.86), xycoords='axes fraction', bbox=dict(boxstyle="round", fc='1', ec='0.75'))

axs[0, 1].annotate('$\mathcal{S}_1$', xy=(0.05, 0.86), xycoords='axes fraction', bbox=dict(boxstyle="round", fc='1', ec='0.75'))

axs[1, 0].annotate('$\mathcal{S}_3$', xy=(0.05, 0.86), xycoords='axes fraction', bbox=dict(boxstyle="round", fc='1', ec='0.75'))

axs[1, 1].annotate('$\mathcal{S}_3$', xy=(0.05, 0.86), xycoords='axes fraction', bbox=dict(boxstyle="round", fc='1', ec='0.75'))

if export_image:
    plt.savefig('BUP.png', bbox_inches='tight', transparent=True)
