#include "sad.hh"

#include <array>
#include <cmath>
#include <fstream>
#include <iostream>
#include <tuple>

using namespace StochasticDifferentiation;
using Real = SAD<double>;

Real f(double p_)
{
  StochasticProgram<double> sp; 
  sp.jumpWeight = 0.5;

  int n = 100;
  
  Real p(&sp,p_,1);
  // return p+p;

  if (false)
  {
    Real b = bernoulli(p);
    for (int i=1; i<n; ++i)
      b += bernoulli(p);
    return b;
  }
  else
  {
    Real b = binomial(p,n,false);
    return b;
  }

  //   Real b3 = bernoulli(2.0*p);
  // Real n = gauss(p,Real(&sp,1,0));
  // Real b2 = bernoulli(p);
  // Real b3 = bernoulli(p);
}

// Compute mean and sample variance of value and of derivative
std::array<double,4> favg(double p, int n=10000)
{
    std::vector<Real> fs;
    for (int i=0; i<n; ++i)
        fs.push_back(f(p));
    double fmean=0, fstd=0, dfmean=0, dfstd=0;
    for (Real const& r: fs)
    {
        fmean += r.value;
        dfmean += r.derivativeSample();
    }
    fmean /= n;
    dfmean /= n;

    for (Real const& r: fs)
    {
        double a = r.value-fmean;
        fstd += a*a;
        double b = r.derivativeSample()-dfmean;
        dfstd += b*b;
    }
    fstd = std::sqrt(fstd/n);
    dfstd = std::sqrt(dfstd/n);

    return std::array<double,4>{fmean,dfmean,fstd,dfstd};
}

int main(void)
{
    std::ofstream out("out.gnu");

    double prev = std::nan("");
    double inc = 0.02;
    int n = 50000;
    for (double p=0; p<=1; p+=inc)
    {
        auto ff = favg(p,n);
        // 1: p, 2: E[f], 3: E[f'], 4: E[f'] finite diff, 5: E[f]-sigma, 6: E[f]+sigma
        // 7: E[df]-sigma, 8: E[df]+sigma
        out << p << " " << ff[0] << " " << ff[1] << " "
            << (ff[0]-prev)/inc << " "
            << ff[0]-ff[2]/std::sqrt(n) << " " << ff[0]+ff[2]/std::sqrt(n) << " "
            << ff[1]-ff[3]/std::sqrt(n) << " " << ff[1]+ff[3]/std::sqrt(n) << "\n";
        prev = ff[0];
    }
}
